@@ -1,5 +1,5 @@
 /* 
- * GNetwork Library: libgnetwork/gnetwork-utils.c
+ * GNetwork Library: libgnetwork/gnetwork-interfaces.c
  *
  * Copyright (C) 2003 James M. Cape
  *
@@ -63,7 +63,7 @@ struct _GNetworkInterfaceInfo
 
   gint ref;
 
-  guint index_;
+  guint64 index_;
   gchar *name;
 
   GNetworkIpAddress ip4_address;
@@ -137,7 +137,7 @@ append_iface_info_from_interface (GNetworkInterfaceInfo * info, struct ifaddrs *
     case AF_PACKET:
       info->protocols |= GNETWORK_PROTOCOL_PACKET;
 
-      info->index_ = ((struct sockaddr_ll *) data->ifa_addr)->sll_ifindex;
+      info->index_ = ((struct sockaddr_ll *) (data->ifa_addr))->sll_ifindex;
 
       info->hw_address = _gnetwork_sockaddr_get_address (data->ifa_addr);
       info->hw_broadcast_or_destination = _gnetwork_sockaddr_get_address (data->ifa_dstaddr);
