@@ -40,6 +40,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -208,8 +209,13 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
       sa = g_malloc0 (sa_size);
 
       errno = 0;
+#ifdef MSG_NOSIGNAL
       bytes_done = recvfrom (udp->_priv->sockfd, buffer, udp->_priv->buffer_size, MSG_NOSIGNAL,
 			     sa, &sa_size);
+#else
+      bytes_done = recvfrom (udp->_priv->sockfd, buffer, udp->_priv->buffer_size, 0,
+	      		    sa, &sa_size);
+#endif
       en = errno;
 
       target->hostname = NULL;
@@ -281,8 +287,13 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
 						 &sa_size);
 
 	  errno = 0;
+#ifdef MSG_NOSIGNAL
 	  bytes_done = sendto (udp->_priv->sockfd, item->data, item->length, MSG_NOSIGNAL, sa,
 			       sa_size);
+#else
+	  bytes_done = sendto (udp->_priv->sockfd, item->data, item->length, 0, sa,
+		  	       sa_size);
+#endif
 	  en = errno;
 	  g_free (sa);
 
@@ -1095,7 +1106,7 @@ gnetwork_udp_datagram_class_init (GNetworkUdpDatagramClass * class)
   g_object_class_install_property (object_class, BROADCAST,
 				   g_param_spec_boolean ("broadcast", _("Allow Broadcasting"),
 							 _("Whether or not to allow broadcasting "
-							   "through this datagram."), FALSE,
+							   "through this socket."), FALSE,
 							 (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
   g_object_class_install_property (object_class, INTERFACE,
 				   g_param_spec_string ("interface", _("Local Interface"),
