@@ -200,7 +200,7 @@ gnetwork_unix_connection_close (GNetworkUnixConnection * connection)
   g_object_freeze_notify (G_OBJECT (connection));
   g_object_notify (G_OBJECT (connection), "status");
   g_object_notify (G_OBJECT (connection), "unix-status");
-  g_object_notify (G_OBJECT (connection), "socket-fd");
+  g_object_notify (G_OBJECT (connection), "socket");
   g_object_thaw_notify (G_OBJECT (connection));
 }
 
@@ -230,7 +230,7 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUnixConnect
       gsize bytes_read;
       GError *error = NULL;
 
-      buffer = g_new0 (guchar, connection->_priv->buffer_size + 1);
+      buffer = g_new (guchar, connection->_priv->buffer_size + 1);
       
       status = g_io_channel_read_chars (channel, buffer, connection->_priv->buffer_size,
 					&bytes_read, &error);
@@ -568,6 +568,7 @@ gnetwork_unix_connection_send (GNetworkUnixConnection * connection, gconstpointe
 
   item = g_new (BufferItem, 1);
   item->data = g_new (guchar, length + 1);
+  G_UCHAR (item->data)[length] = 0;
   memcpy (item->data, data, length);
   item->length = length;
 
@@ -784,6 +785,7 @@ gnetwork_unix_connection_instance_init (GNetworkUnixConnection * connection)
   connection->_priv->unix_status = GNETWORK_UNIX_CONNECTION_CLOSED;
 
   connection->_priv->filename = NULL;
+  connection->_priv->buffer = NULL;
 
   connection->_priv->channel = NULL;
   connection->_priv->source_id = 0;
