@@ -30,8 +30,13 @@
 #include <errno.h>
 
 #include <net/if.h>
+#ifdef __linux__
 #include <netpacket/packet.h>
 #include <netinet/ether.h>
+#else
+#include <net/ethernet.h>
+#endif
+#include <netinet/in.h>
 #include <arpa/inet.h>
 
 #include <unistd.h>
@@ -269,6 +274,7 @@ _gnetwork_sockaddr_get_address (const struct sockaddr *sock)
 	  inet_ntop (sock->sa_family, &(((struct sockaddr_in6 *) sock)->sin6_addr), address,
 		     INET_ADDRESS_SIZE);
 	  break;
+#ifdef AF_PACKET
 	case AF_PACKET:
 	  switch (((struct sockaddr_ll *) sock)->sll_halen)
 	    {
@@ -285,6 +291,7 @@ _gnetwork_sockaddr_get_address (const struct sockaddr *sock)
 	      break;
 	    }
 	  break;
+#endif
 	}
 
       return (address[0] != '\0' ? g_strdup (address) : NULL);
@@ -335,9 +342,11 @@ _gnetwork_get_socket_protocol (gpointer sockfd)
 	case AF_INET6:
 	  retval = GNETWORK_PROTOCOL_IPv6;
 	  break;
+#ifdef AF_PACKET
 	case AF_PACKET:
 	  retval = GNETWORK_PROTOCOL_PACKET;
 	  break;
+#endif
 	default:
 	  break;
 	}
